/*
 * Project Name:
     T6963C_240x64 (GLCD Library demo for Toshiba's T6963 Controller)
 * Copyright:
     (c) Mikroelektronika, 2005-2010.
 * Revision History:
     20080930:
       - initial release; Credits to Bruno Gavand.
     20101026 (TL):
       - adapted for PRO compilers.
 * Description:
     This code in intended to work with GLCD's based on TOSHIBA T6963C controller.
     Pressing buttons PB0 .. PB5 generates commands for text and graphic displaying.
     This parts may need a -15V power supply on Vee for LCD drive,
     a simple DC/DC converter can be made with a 2N2905, 220 H self, diode & 47 F cap,
     transistor base is driven with PWM thrue a current limiting resistor.
     This parts have a 8 Kb built-in display RAM, this allows 2 graphics panels
     and one text panel.
 * Test configuration:
     MCU:             ATmega16
                      http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf
     Dev.Board:       EasyAVR6
                      http://www.mikroe.com/eng/products/view/321/easyavr6-development-system/
     Oscillator:      External Clock 8.0000 MHz
     Ext. Modules:    ac:GLCD_240x64_Adapter_Board on PORTC(Control) and PORTD(Data)
                      http://www.mikroe.com/eng/products/view/144/glcd-240x64-adapter-board/
                      T6963C display 240x64 pixels
                      http://www.mikroe.com/eng/products/view/277/various-components/#other
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/
 * NOTES:
     - Put button jumper (J13) into Vcc position and pull-down PORTA.
 */

#include        "__T6963C.h"

// T6963C module connections
char T6963C_dataPort at PORTD;                   // DATA port
char T6963C_dataPort_Direction at DDRD;          // DATA direction register

sbit T6963C_ctrlwr  at PORTC0_bit;               // WR write signal
sbit T6963C_ctrlrd  at PORTC1_bit;               // RD read signal
sbit T6963C_ctrlcd  at PORTC3_bit;               // CD command/data signal
sbit T6963C_ctrlrst at PORTC4_bit;               // RST reset signal
sbit T6963C_ctrlwr_Direction  at DDC0_bit;       // WR write signal
sbit T6963C_ctrlrd_Direction  at DDC1_bit;       // RD read signal
sbit T6963C_ctrlcd_Direction  at DDC3_bit;       // CD command/data signal
sbit T6963C_ctrlrst_Direction at DDC4_bit;       // RST reset signal

// Signals not used by library, they are set in main function
sbit T6963C_ctrlce at PORTC2_bit;                // CE signal
sbit T6963C_ctrlfs at PORTC5_bit;                // FS signal
sbit T6963C_ctrlce_Direction at DDC2_bit;        // CE signal direction
sbit T6963C_ctrlfs_Direction at DDC5_bit;        // FS signal direction
// End T6963C module connections

/*
 * bitmap pictures stored in ROM
 */
const code char mikroE_240x64_bmp[];
const code char einstein[];

void main() {

  //#define LINE_DEMO           // Uncomment to demonstrate line drawing routines
  //#define FILL_DEMO           // Uncomment to demonstrate fill routines
  //#define PARTIAL_IMAGE_DEMO  // Uncomment to demonstrate partial image routine

  char txt1[] = " EINSTEIN WOULD HAVE LIKED mE";
  char txt[] =  " GLCD LIBRARY DEMO, WELCOME !";
  char txt2[] = "Partial image demo!";

  unsigned char  panel;         // Current panel
  unsigned int   i;             // General purpose register
  unsigned char  curs;          // Cursor visibility
  unsigned int   cposx, cposy;  // Cursor x-y position
  unsigned short s;             // counter

  DDA0_bit = 0;                 // Set PA0 as input
  DDA1_bit = 0;                 // Set PA1 as input
  DDA2_bit = 0;                 // Set PA2 as input
  DDA3_bit = 0;                 // Set PA3 as input
  DDA4_bit = 0;                 // Set PA4 as input
  DDA5_bit = 0;                 // Set PA5 as input

  T6963C_ctrlce_Direction = 1;
  T6963C_ctrlce = 0;             // Enable T6963C
  T6963C_ctrlfs_Direction = 1;
  T6963C_ctrlfs = 0;             // Font Select 8x8

  // Initialize T6963C
  T6963C_init(240, 64, 8);

  /*
   * Enable both graphics and text display at the same time
   */
  T6963C_graphics(1);
  T6963C_text(1);

  panel = 0;
  i = 0;
  curs = 0;
  cposx = cposy = 0;

  /*
   * Text messages
   */
  T6963C_write_text(txt, 0, 0, T6963C_ROM_MODE_XOR);
  T6963C_write_text(txt1, 0, 7, T6963C_ROM_MODE_XOR);

  /*
   * Cursor
   */
  T6963C_cursor_height(8);       // 8 pixel height
  T6963C_set_cursor(0, 0);       // Move cursor to top left
  T6963C_cursor(0);              // Cursor off

  /*
   * Draw solid boxes
   */
  T6963C_box(0, 0, 239, 8, T6963C_WHITE);
  T6963C_box(0, 55, 239, 63, T6963C_WHITE);

  /*
   * Draw rectangles
   */
  #ifdef LINE_DEMO
    T6963C_rectangle(0, 0, 239, 63, T6963C_WHITE);
    T6963C_rectangle(20, 11, 219, 53, T6963C_WHITE);
    T6963C_rectangle(40, 21, 199, 43, T6963C_WHITE);
    T6963C_rectangle(60, 30, 179, 34, T6963C_WHITE);
  #endif

  /*
   * Draw rounded edge rectangle
   */
  #ifdef LINE_DEMO
    T6963C_Rectangle_Round_Edges(30,16, 209, 48, 12, T6963C_WHITE);
    T6963C_Rectangle_Round_Edges(50,25, 189, 39, 5, T6963C_WHITE);
  #endif

  /*
   * Draw filled rounded edge rectangle
   */
  #ifdef FILL_DEMO
    T6963C_Rectangle_Round_Edges_Fill(10, 10, 229, 53, 12, T6963C_WHITE);
    T6963C_Rectangle_Round_Edges_Fill(15, 15, 224, 49, 12, T6963C_BLACK);
    T6963C_Rectangle_Round_Edges_Fill(20, 20, 219, 44, 9, T6963C_WHITE);
    T6963C_Rectangle_Round_Edges_Fill(25, 25, 214, 39, 7, T6963C_BLACK);
    T6963C_Rectangle_Round_Edges_Fill(30, 30, 209, 34, 2, T6963C_WHITE);
  #endif

  /*
   * Draw a cross
   */
  #ifdef LINE_DEMO
    T6963C_line(0, 0, 239, 63, T6963C_WHITE);
    T6963C_line(0, 63, 239, 0, T6963C_WHITE);
  #endif

  /*
   * Draw circles
   */
  #ifdef LINE_DEMO
    T6963C_circle(120, 32, 10, T6963C_WHITE);
    T6963C_circle(120, 32, 30, T6963C_WHITE);
    T6963C_circle(120, 32, 50, T6963C_WHITE);
    T6963C_circle(120, 32, 70, T6963C_WHITE);
    T6963C_circle(120, 32, 90, T6963C_WHITE);
    T6963C_circle(120, 32, 110, T6963C_WHITE);
    T6963C_circle(120, 32, 130, T6963C_WHITE);
  #endif

  /*
   * Draw filled circles
   */
  #ifdef FILL_DEMO
    T6963C_circle_fill(120, 32, 40, T6963C_WHITE);
    T6963C_circle_fill(120, 32, 35, T6963C_BLACK);
    T6963C_circle_fill(120, 32, 30, T6963C_WHITE);
    T6963C_circle_fill(120, 32, 25, T6963C_BLACK);
    T6963C_circle_fill(120, 32, 20, T6963C_WHITE);
    T6963C_circle_fill(120, 32, 15, T6963C_BLACK);
    T6963C_circle_fill(120, 32, 10, T6963C_WHITE);
    T6963C_circle_fill(120, 32, 5, T6963C_BLACK);
  #endif

  Delay_ms(1000);
  T6963C_sprite(60, 0, einstein, 120, 64);     // Draw a sprite
  Delay_ms(1000);

  T6963C_setGrPanel(1);                        // Select other graphic panel

  T6963C_image(mikroE_240x64_bmp);
  T6963C_displayGrPanel(1);
  Delay_ms(1000);
  #ifdef PARTIAL_IMAGE_DEMO 
    T6963C_grFill(0);
    T6963C_PartialImage(0, 0, 64, 64, 240, 64, mikroE_240x64_bmp);   // Display partial image
    Delay_ms(1000);
    T6963C_graphics(0);
  #endif
  T6963C_image(mikroE_240x64_bmp);
  T6963C_graphics(1);
  T6963C_displayGrPanel(0);

  for(;;) {                                    // Endless loop
 
    /*
     * If PA0 is pressed, display only graphic panel
     */
    if(PINA0_bit) {
      T6963C_graphics(1);
      T6963C_text(0);
      Delay_ms(300);
      }

    /*
     * If PA1 is pressed, toggle the display between graphic panel 0 and graphic panel 1
     */
    else if(PINA1_bit) {
      panel++;
      panel &= 1;
      T6963C_displayGrPanel(panel);
      Delay_ms(300);
      }

    /*
     * If PA2 is pressed, display only text panel
     */
    else if(PINA2_bit) {
      T6963C_graphics(0);
      T6963C_text(1);
      Delay_ms(300);
      }

    /*
     * If PA3 is pressed, display text and graphic panels
     */
    else if(PINA3_bit) {
      T6963C_graphics(1);
      T6963C_text(1);
      Delay_ms(300);
      }

    /*
     * If PA4 is pressed, change cursor
     */
    else if(PINA4_bit) {
      curs++;
      if(curs == 3) curs = 0;
      switch(curs) {
        case 0:
          // no cursor
          T6963C_cursor(0);
          break;
        case 1:
          // blinking cursor
          T6963C_cursor(1);
          T6963C_cursor_blink(1);
          break;
        case 2:
          // non blinking cursor
          T6963C_cursor(1);
          T6963C_cursor_blink(0);
          break;
        }
      Delay_ms(300);
      }

    #ifdef PARTIAL_IMAGE_DEMO
     /*
     * If PA5 is pressed, perform the "Partial image" demostration
     */
    else if(PINA5_bit) {
      T6963C_setGrPanel(0);
      T6963C_setTxtPanel(0);
      T6963C_txtFill(0);
      T6963C_setGrPanel(1);
      T6963C_setTxtPanel(0);
      T6963C_graphics(1);
      T6963C_text(1);
      T6963C_displayGrPanel(1);
      T6963C_write_text(txt2, 5, 7, T6963C_ROM_MODE_XOR);
      Delay_1sec();

      for(s = 0; s < 3; s++){
         T6963C_grFill(0);
         T6963C_PartialImage(s*80 + 10, 0, 64, 64, 240, 64, mikroE_240x64_bmp); // Partial image
         Delay_ms(500);
       }

      T6963C_txtFill(0);
      T6963C_image(mikroE_240x64_bmp);
      T6963C_write_text(txt, 0, 0, T6963C_ROM_MODE_XOR);
      T6963C_write_text(txt1, 0, 7, T6963C_ROM_MODE_XOR);
    }
    #endif

    /*
     * Move cursor, even if not visible
     */
    cposx++;
    if(cposx == T6963C_txtCols) {
      cposx = 0;
      cposy++;
      if(cposy == T6963C_grHeight / T6963C_CHARACTER_HEIGHT) {
        cposy = 0;
        }
      }
    T6963C_set_cursor(cposx, cposy);

    Delay_ms(100);
    }
}